package net.moonleay.bedge.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.application.slash.converters.impl.numberChoice
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import dev.kord.core.behavior.interaction.followup.edit
import dev.kord.rest.builder.message.create.actionRow
import dev.kord.rest.builder.message.create.embed
import dev.kord.rest.builder.message.modify.embed
import net.moonleay.bedge.data.database.repository.UserRepository
import net.moonleay.bedge.util.*

class RequestStreakFix : Extension() {

    override val name = "requeststreakfix"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        publicSlashCommand(::RequestStreakFixArguments) {
            name = "requeststreakfix"
            description = "Request a streak fix. This will be voted on. atleast 3 votes are needed to fix the streak."

            this.action {
                val target = this.user.asUser()
                val wantedStreak = this.arguments.amountOfDays
                if (!UserRepository.doesUserExist(target.id.value)) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "User not found!",
                                "You cant fix what ain't broken. You did not use the bot yet, so there is no information about you.",
                                target.username,
                            )
                        )
                    }
                    return@action
                }
                val tData = UserRepository.getUserByID(target.id.value)!!
                if (!tData.isAwake) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "You are already asleep!",
                                "You can't fix a streak if you are already asleep.",
                                target.username,
                            )
                        )
                    }
                    return@action
                }
                if (tData.currentStreak != 0) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "Your streak is not broken!",
                                "You can't fix a streak if it is not broken.\n(Your current streak has to be 0.)",
                                target.username,
                            )
                        )
                    }
                    return@action
                }
                val eb = MessageUtil.getEmbedWithTable(
                    EmbedColor.INFO,
                    "Fix ${target.username}s streak",
                    "${target.mention} wants to fix their streak. Please vote on this request.\n" +
                            "The wanted streak is ${wantedStreak}.",
                    mapOf(
                        "Vouched" to listOf(),
                        "Declined" to listOf(),
                    )
                )
                val msg = this.respond {
                    this.content = "@everyone"
                    this.embed {
                        this.author {
                            this.name = "Requested days: $wantedStreak"
                        }
                        this.color = eb.color
                        this.title = eb.title
                        this.description = eb.description
                        this.fields = eb.fields
                        this.footer {
                            this.icon = target.avatar?.cdnUrl?.toUrl()
                            this.text = "requested by ${target.username} {${target.id.value}} [ongoing]"
                        }
                    }

                    this.actionRow {
                        this.components.addAll(EmbedUtil.getStreakFixButtons().components)
                    }
                }
            }
        }
    }

    inner class RequestStreakFixArguments : Arguments() {

        val amountOfDays by numberChoice {
            this.name = "amountofdays"
            this.description = "The desired amount of days for the streak."
        }
    }
}
