/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.util

import dev.kord.common.entity.ButtonStyle
import dev.kord.core.entity.Embed
import dev.kord.rest.builder.component.ActionRowBuilder
import dev.kord.rest.builder.message.EmbedBuilder

object EmbedUtil {
    fun getStreakFixButtons(): ActionRowBuilder {
        val ar = ActionRowBuilder()
        ar.interactionButton(ButtonStyle.Success, "public.edit.btn.streakfix.vouch") {
            this.label = "Vouch"
        }
        ar.interactionButton(ButtonStyle.Danger, "public.edit.btn.streakfix.decline") {
            this.label = "Decline"
        }
        return ar
    }

    fun replaceXWithYinValuesAtTable(x: String, y: String, e: Embed, table: Int): EmbedBuilder {
        return replaceXWithYinValuesAtTable(x, y, MessageUtil.getAClonedEmbed(e), table)
    }

    fun replaceXWithYinValuesAtTable(x: String, y: String, ebb: EmbedBuilder, table: Int): EmbedBuilder {
        val ebbb = MessageUtil.getAClonedEmbed(ebb)
        ebbb.fields = mutableListOf()
        for ((i, f) in ebb.fields.withIndex()) {
            val fb = EmbedBuilder.Field()
            fb.name = f.name
            if (i == table - 1) {
                val v = f.value.split("\n").toMutableList()
                for ((j, l) in v.withIndex())
                    if (l.contains(x))
                        v[j] = y
                v.removeIf {
                    !it.contains("@")
                }
                fb.value = v.joinToString("\n")
            } else
                fb.value = f.value
            fb.inline = true
            ebbb.fields.add(fb)
        }
        return ebbb
    }

    fun getAmountOfUsersInTableX(e: EmbedBuilder, table: Int): Int {
        val f = e.fields[table - 1]
        if (!f.value.contains("@"))
            return 0
        return f.value.split("\n").toMutableList().size
    }

    fun getAllUsersInTheFirstXTables(amountOfTables: Int, e: Embed): List<String> {
        val users = mutableListOf<String>()
        for (i in 0 until amountOfTables - 1) {
            val f = e.fields[i]
            if (!f.value.contains("@"))
                continue // check next one. this one does not have any entries
            val v = f.value.split("\n").toMutableList()
            for (l in v) {
                Logger.out(l)
                users.add(l.subSequence(2, l.indexOf(">")).toString())
            }
        }
        return users
    }

    fun addXToValuesAtTable(x: String, e: Embed, table: Int): EmbedBuilder {
        return addXToValuesAtTable(x, MessageUtil.getAClonedEmbed(e), table)
    }

    fun addXToValuesAtTable(x: String, ebb: EmbedBuilder, table: Int): EmbedBuilder {
        val ebbb = MessageUtil.getAClonedEmbed(ebb)
        ebbb.fields = mutableListOf()
        ebb.fields.forEachIndexed { i, f ->
            val fb = EmbedBuilder.Field()
            fb.name = f.name
            if (i == table - 1)
                fb.value = f.value + "\n<@$x>"
            else {
                val v = f.value.split("\n").toMutableList()
                for ((j, l) in v.withIndex())
                    if (l.contains(x))
                        v[j] = ""
                v.removeIf {
                    !it.contains("@")
                }
                fb.value = v.joinToString("\n")
            }

            fb.inline = true
            ebbb.fields.add(fb)
        }
        return ebbb
    }

}
